/*****************************************************************************
* FILENAME
*   timers.c
*
* DESCRIPTION
*   Using the on chip timers Timer 0 and Timer 1 
*
*  Rev 1.00   R.Sikora  
*
*-----------------------------------------------------------------------------
*  HISTORY
*  Rev 1.00  22 February 2002. Created by R.Sikora. 
*     
*
******************************************************************************/

/* c6x.h contains declarations for IER, ICR, ISR, CSR etc */
/* These use the instruction MVC because they are control registers */
/* They are not declared as *(volatile int*)ICR! */


#include <c6x.h>
#include "c6211dsk.h"            
#include "timers.h"
#include "interrupts.h"

volatile int timer0_flag; /* IMPORTANT. MUST BE VOLATILE */
volatile int timer1_flag;

/* Various bits of timer control registers TIMER0_CTRL */

/*-------------------------------------------------------------------------*/
/* timer0_init() - used to initialize TIMER0                               */
/*-------------------------------------------------------------------------*/
/*                                                                         */
/* INPUT: period of timer.                                                 */
/*                                                                         */
/* TIMER0_PRD = 0xFFFFFF will give about 2 flashes per second              */
/* TIMER0_PRD = 1875 gives 10.00 kHz                                       */
/* TIMER0_PRD = 18750 gives 1.00 kHz                                       */
/* TIMER0_PRD = 187500 gives 100 Hz                                        */
/*                                                                         */
/*-------------------------------------------------------------------------*/

void timer0_init(unsigned int period)
{
   *(volatile unsigned int *) TIMER0_CTRL &= 0xFF3F;     /* Hold the timer        */
   *(volatile unsigned int *) TIMER0_CTRL |= 0x0200;     /* Use CPU CLK/4         */   
   *(volatile unsigned int *) TIMER0_COUNT = 0xFFFFFFFF; /* Ensure timer starts in right place ? */
   *(volatile unsigned int *) TIMER0_PRD   = period;     /* Set timer period */
   *(volatile unsigned int *) TIMER0_CTRL |= 0x03C1;     /* start the timer       */
                                                         /* Enable timer 0 interrupt? */
}                                                        /* Timer output on pin T1OUT */

/*-------------------------------------------------------------------------*/
/* timer1_init() - used to initialize TIMER1                               */
/*-------------------------------------------------------------------------*/

/* TIMER1_PRD = 0xFFFFFF will give about 2 flashes per second */
/* TIMER1_PRD = 1875 gives 10.00 kHz */
/* TIMER1_PRD = 2234 gives 8.00 kHz */
/* TIMER1_PRD = 18750 gives 1.00 kHz */

void timer1_init(unsigned int period)
{
   *(volatile unsigned int *) TIMER1_CTRL &= 0xFF3F;     /* hold the timer        */
   *(volatile unsigned int *) TIMER1_CTRL |= 0x0200;     /* use CPU CLK/4         */   
   *(volatile unsigned int *) TIMER1_COUNT = 0xFFFFFFFF; /* Ensure timer starts in right place ? */
   *(volatile unsigned int *) TIMER1_PRD   = period;     /* Set period */
   *(volatile unsigned int *) TIMER1_CTRL |= 0x03C1;     /* start the timer       */
                                                         /* Enable timer 0 interrupt? */
} 

/*-------------------------------------------------------------------------*/
/* timer0_isr() - interrupt service routine for TIMER0                     */
/*-------------------------------------------------------------------------*/

/* Note that the isr is deliberately kept short */
/* Does not seem to need ICR |= INT14 to clear. Check in book */

interrupt void timer0_isr()
{
      timer0_flag = 1;     /* Set the flag */
}

/*-------------------------------------------------------------------------*/
/* timer1_isr() - interrupt service routine for TIMER0                     */
/*-------------------------------------------------------------------------*/

/* Note that the isr is deliberately kept short */
/* Does not seem to need ICR |= INT14 to clear. Check in book */

interrupt void timer1_isr()
{
      timer1_flag = 1;     /* Set the flag */
}

/*-------------------------------------------------------------------------*/
/* End of timers.c                                                          */
/*-------------------------------------------------------------------------*/

